class AttributeModifier:

    def __init__(self, attr_name):
        self._attr_name = attr_name

    def __call__(self, obj):
        return getattr(obj, self._attr_name)


class AttributesModifier:

    def __init__(self, attr_names):
        self._attr_names = attr_names

    def __call__(self, obj):
        return tuple(getattr(obj, attr_name) for attr_name in self._attr_names)



from index_by.attr import index_by_attr, index_by_attr_as_list, index_by_attrs, index_by_attrs_as_list
from index_by.key import index_by_key, index_by_key_as_list, index_by_keys, index_by_keys_as_list
from index_by.modifier import index_by_modifier, index_by_modifier_as_list
from index_by.pop_key import index_by_pop_key, index_by_pop_key_as_list, index_by_pop_keys, index_by_pop_keys_as_list

__all__ = [
    'index_by_attr',
    'index_by_attr_as_list',
    'index_by_attrs',
    'index_by_attrs_as_list',
    'index_by_key',
    'index_by_key_as_list',
    'index_by_keys',
    'index_by_keys_as_list',
    'index_by_pop_key',
    'index_by_pop_key_as_list',
    'index_by_pop_keys',
    'index_by_pop_keys_as_list',
    'index_by_modifier',
    'index_by_modifier_as_list'
]



def index_by_modifier(sequence, modifier, target_index=None):
    """
    Create an index of objects in a sequence. The index's key will be provider by the modifier. Values are single
    objects, so in case of a duplicate key the latest object will persist.

    The modifier is called for each object with that object as an argument and is supposed to
    return the key to be used for that object in the index.

    :param sequence: sequence of objects (list, tuple or otherwise iterable object)
    :param modifier: callable that takes an object and returns a hashable value.
    :param target_index: dict that will be populated with objects from the sequence (optional)
    :return: dict
    """
    index = target_index if target_index is not None else dict()
    for element in sequence:
        key = modifier(element)
        index[key] = element
    return index


def index_by_modifier_as_list(sequence, modifier, target_index=None):
    """
    Create an index of objects in a sequence. The index's key will be provider by the modifier. Values are lists,
    so keys are not expected to be unique.

    The modifier is called for each object with that object as an argument and is supposed to
    return the key to be used for that object in the index.

    :param sequence: sequence of objects (list, tuple or otherwise iterable object)
    :param modifier: callable that takes an object and returns a hashable value.
    :param target_index: dict that will be populated with objects from the sequence (optional)
    :return: dict
    """
    index = target_index if target_index is not None else dict()
    for element in sequence:
        key = modifier(element)
        index.setdefault(key, [])
        index[key].append(element)
    return index
